# Generated by Django 3.2.7 on 2021-10-17 21:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('workspaces', '0005_auto_20211017_2104'),
    ]

    operations = [
        migrations.AddField(
            model_name='environmentconfig',
            name='workspace',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='workspaces.workspace'),
        ),
        migrations.AlterUniqueTogether(
            name='environmentconfig',
            unique_together={('key', 'workspace')},
        ),
    ]



from django.apps import AppConfig
from django.db.models.signals import post_migrate


class AuthorizationConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "munity.authorization"

    def ready(self):
        from .models import sync_permissions
        post_migrate.connect(sync_permissions, sender=self)



# Generated by Django 3.2.7 on 2021-12-02 21:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('generic_groups', '0006_alter_genericgroup_workspace'),
        ('files', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='file',
            name='generic_groups',
            field=models.ManyToManyField(blank=True, to='generic_groups.GenericGroup'),
        ),
    ]



from django_filters import rest_framework as filters
from munity.views import MunityViewSet
from rest_framework import serializers
from pprint import pprint

from .models import File

class FileSerializer(serializers.ModelSerializer):
    class Meta:
        fields = [
            "id",
            "name",
            "file",
            "type",
            "size",
            "workspace",
            "created",
            "modified",
        ]
        model = File
    def create(self, validated_data):
        file = validated_data.get('file')
        validated_data['size'] = file.size
        validated_data['name'] = file.name
        validated_data['type'] = file.content_type
        return File.objects.create(**validated_data)

    def update(self, instance, validated_data):
        return instance.update(**validated_data)


class FileFilter(filters.FilterSet):
    class Meta:
        fields = {
            "id": ["exact", "in"],
            "name": ["exact", "in", "contains"],
            "workspace": ["exact", "in"],
            "type": ["exact", "in", "contains"],
            "size": ["gt", "gte", "lt", "lte"],
            "created": ["gt", "gte", "lt", "lte"],
            "modified": ["gt", "gte", "lt", "lte"],
        }
        model = File


class FileViewSet(MunityViewSet):
    serializer_class = FileSerializer
    filterset_class = FileFilter


# Generated by Django 3.2.7 on 2021-10-17 21:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('authorization', '0003_auto_20211017_2031'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='permission',
            name='workspace',
        ),
        migrations.RemoveField(
            model_name='role',
            name='workspace',
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-18 00:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('generic_groups', '0005_genericgroup_workspace'),
        ('authorization', '0005_auto_20211017_2105'),
        ('users', '0007_auto_20211018_0049'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserRoleGenericGroup',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='generic_groups.genericgroup')),
                ('role', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='authorization.role')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'unique_together': {('user', 'group', 'role')},
            },
        ),
    ]



from django_extensions.db.models import TimeStampedModel
from django.db import models

from ..models import MunityModel

class Settings(MunityModel):
    key = models.CharField(max_length=100)
    value = models.JSONField()

    def __str__(self):
        return f"{self.key}@{self.workspace or 'overmind'}"

    class Meta:
        unique_together = [("key", "workspace")]



# Generated by Django 3.2.7 on 2021-10-17 23:01

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('authorization', '0005_auto_20211017_2105'),
        ('users', '0005_user_workspace'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='role',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='authorization.role'),
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-17 21:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('workspaces', '0006_auto_20211017_2105'),
        ('users', '0004_remove_user_workspace'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='workspace',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='workspaces.workspace'),
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-20 00:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('records', '0007_alter_record_workspace'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='record',
            name='model_name',
        ),
    ]



from django_filters import rest_framework as filters
from rest_framework import serializers, viewsets
from ..views import MunityViewSet

from .models import Record

class RecordSerializer(serializers.ModelSerializer):
    class Meta:
        fields = [
            "id",
            "dif_value",
            "next_value",
            "created",
            "user",
            "product_object_id",
            "product_content_type",
            "action",
        ]
        model = Record

class RecordsFilter(filters.FilterSet):
    class Meta:
        fields = {
            "created": ["gt", "gte", "lt", "lte"],
            "user": ["exact"],
        }
        model = Record


class RecordsViewSet(viewsets.ModelViewSet):

    def get_queryset(self):
        model = self.serializer_class.Meta.model

        # filter request to current model workspace
        if "workspace_pk" in self.kwargs:
            return model.objects.filter(workspace=self.kwargs["workspace_pk"])
        return model.objects.all()

    serializer_class = RecordSerializer
    filterset_class = RecordsFilter




# Generated by Django 3.2.7 on 2021-12-15 16:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0003_settings_generic_groups'),
    ]

    operations = [
        migrations.AddField(
            model_name='settings',
            name='disabled',
            field=models.BooleanField(default=False),
        ),
    ]



import uuid

from django.contrib.auth.models import AbstractUser
from django.db import models
from django.db.models.deletion import SET_NULL
from django.db.models import F, Func
from django_extensions.db.models import TimeStampedModel
from ..models import MunityModel


class User(AbstractUser, MunityModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    phone_number = models.CharField(max_length=20, blank=True, null=True, default=None)
    avatar = models.ForeignKey("files.File", related_name="avatars", on_delete=models.CASCADE, blank=True, null=True, default=None)
    # user are related to workspace by role., dont use munitymodel workspaces
    workspace = None
    disabled = models.BooleanField(default=False)
    has_overmind_access = models.BooleanField(default=False, blank=True)

    class Meta:
        ordering = (Func(F("username"), function='Lower'),)
    def get_absolute_url(self):
        return f"/users/{self.id}"

class UserRoleWorkspace(TimeStampedModel):
    user = models.ForeignKey("users.User", related_name="user_role_workspaces", on_delete=models.CASCADE)
    workspace = models.ForeignKey("workspaces.Workspace", on_delete=models.CASCADE, null=True)
    role = models.ForeignKey("authorization.Role", on_delete=models.CASCADE)

    class Meta:
        unique_together=('user', 'workspace')

class UserRoleGenericGroup(TimeStampedModel):
    user = models.ForeignKey("users.User", on_delete=models.CASCADE)
    group = models.ForeignKey("generic_groups.GenericGroup", on_delete=models.CASCADE)
    role = models.ForeignKey("authorization.Role", on_delete=models.CASCADE)

    class Meta:
        unique_together=('user', 'group')



from django.contrib.contenttypes.models import ContentType
from django.db import models

from ..models import MunityModel


class Permission(MunityModel):
    ressource = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, related_name="munity_permissions"
    )
    action = models.CharField(max_length=100)

    def __str__(self):
        return f"{self.ressource}:{self.action}"

    class Meta:
        unique_together = [("ressource", "action")]


class Role(MunityModel):
    name = models.CharField(unique=True, max_length=50)
    permissions = models.ManyToManyField(Permission, blank=True)

    def __str__(self):
        if self.workspace:
            # workspace from MunityModel
            return f"self.name[{self.workspace}]"
        return self.name


DEFAULT_ACTIONS = ["list", "retrieve", "update", "create", "delete"]

def sync_permissions(sender, **kwargs):
    for ct in ContentType.objects.exclude(
        app_label__in=["admin", "silk", "sessions", "contenttypes", "auth"]
    ):
        for action in DEFAULT_ACTIONS:
            _obj, _created = Permission.objects.get_or_create(
                action=action, ressource=ct
            )



# Generated by Django 3.2.7 on 2021-11-25 20:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('files', '0001_initial'),
        ('users', '0017_alter_user_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='avatar',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='avatars', to='files.file'),
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-17 20:31

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0002_user_related_workspace'),
    ]

    operations = [
        migrations.RenameField(
            model_name='user',
            old_name='related_workspace',
            new_name='workspace',
        ),
    ]



from django.db.models.query_utils import Q
from munity.authorization.models import Permission
from munity.users.models import UserRoleWorkspace
from rest_framework import permissions

from ..utils import get_request_info

class HasEndpointPermission(permissions.BasePermission):

    def has_permission(self, request, view):
        if not request.user.is_active:
            return False

        # \o\ Super admin has god power /o/
        if request.user.is_superuser:
            return True

        (workspace_slug, ressource, action) = get_request_info(request)

        # We can get list even on overmind, we are limitted from our roles
        if action == 'list':
            return True
        # User with workspace privilege can invite/delete/update user for their workspaces
        if request.user.has_overmind_access and ressource == 'users':
            return True

        # searching for needed permission
        needed_permission = Permission.objects.filter(
            action=action,
            ressource__app_label=ressource
        )

        # we can self edit (view will check for right permission on admin actions)
        if ressource == 'users' and action == 'update':
            if request.data.get("id") == str(request.user.id):
                # wtf try to become admin oO
                if request.data.get("is_superadmin"):
                    return False
                # wtf try to get ownership
                if request.data.get("has_overmind_access"):
                    return False
                # remove role changement for self edition
                del request.data["user_role_workspaces"]
                return True

        # All user can manage files
        if ressource == 'files':
            return True

        # checking if user as at least one role that fit needed permission on wanted workspace or on all workspace
        has_role_permission = UserRoleWorkspace.objects.filter(
            Q(user=request.user) &
            Q(Q(workspace=workspace_slug) | Q(workspace=None)) &
            Q(role__permissions__in=needed_permission)
        )

        return has_role_permission


import uuid

from django.db import models
from django.db.models.deletion import SET_NULL
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django_extensions.db.models import TimeStampedModel

from ..users.models import User
from ..workspaces.models import Workspace

class Record(TimeStampedModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    previous_value = models.JSONField(null=True)
    diff_value = models.JSONField(null=True)
    user = models.ForeignKey(User, on_delete=SET_NULL, null=True)

    workspace = models.ForeignKey(Workspace, on_delete=SET_NULL, null=True)
    product_object_id = models.UUIDField()
    product_content_type = models.ForeignKey(
        ContentType,
        on_delete=models.PROTECT,
    )
    product = GenericForeignKey(
        'product_content_type',
        'product_object_id',
    )
    action = models.CharField(max_length=64, null=False, default="")


# Generated by Django 3.2.7 on 2021-10-24 03:07

from django.db import migrations, models
import munity.views


class Migration(migrations.Migration):

    dependencies = [
        ('generic_groups', '0006_alter_genericgroup_workspace'),
        ('users', '0012_remove_user_testt'),
    ]

    operations = [
        migrations.CreateModel(
            name='UsersViewSet',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('generic_groups', models.ManyToManyField(blank=True, to='generic_groups.GenericGroup')),
            ],
            options={
                'abstract': False,
            },
            bases=(munity.views.MunityViewSet, models.Model),
        ),
    ]



from django.db import models

from ..models import MunityModel

class GenericGroup(MunityModel):
    label = models.CharField(max_length=255)
    class Meta:
        verbose_name="Generic group"
        verbose_name_plural="Generic groups"

    def get_absolute_url(self):
        return f"/generic_groups/{self.id}"

    def __str__(self):
        return self.label



# Generated by Django 3.2.7 on 2021-12-06 01:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('workspaces', '0011_auto_20211206_0106'),
    ]

    operations = [
        migrations.AlterField(
            model_name='workspace',
            name='name',
            field=models.CharField(blank=True, default='', max_length=258),
        ),
    ]



from django_filters import rest_framework as filters
from ..files.views import FileSerializer
from rest_framework import serializers
from django.db.models.query_utils import Q
from .models import User, UserRoleWorkspace
from rest_framework.exceptions import PermissionDenied
from rest_framework.response import Response
from rest_framework import status

from ..views import MunityViewSet


class UserRoleWorkspaceSerializer(serializers.ModelSerializer):
    class Meta:
        fields = ('workspace', 'role')
        model = UserRoleWorkspace

class UserSerializer(serializers.ModelSerializer):
    class Meta:
        fields = [
            "id",
            "username",
            "email",
            "first_name",
            "last_name",
            "created",
            "avatar",
            "is_superuser",
            "has_overmind_access",
            "modified",
            "generic_groups",
            "user_role_workspaces",
            "password",
            "phone_number"
        ]
        model = User
        extra_kwargs = {
            'password': {
                'write_only': True
            }
        }

    password=serializers.CharField(required=False)
    user_role_workspaces = UserRoleWorkspaceSerializer(many=True)

    # We show full avatar in formation on get, on update we only use doc id
    def to_representation(self, instance):
        response = super().to_representation(instance)
        response['avatar'] = FileSerializer(instance.avatar).data if instance.avatar else None
        return response

    def update(self, instance, validated_data):
        if 'user_role_workspaces' in validated_data:
            user_role_workspaces_data = validated_data.pop('user_role_workspaces')

            # add new rights
            user_new_workspaces = []
            for user_role_workspace_data in user_role_workspaces_data:
                workspace = user_role_workspace_data.get("workspace")
                user_new_workspaces.append(workspace)
                UserRoleWorkspace.objects.filter(user=instance).filter(workspace=workspace).delete()
                UserRoleWorkspace.objects.update_or_create(
                    user=instance, **user_role_workspace_data
                )

            # remove revoked rights
            UserRoleWorkspace.objects.filter(user=instance).exclude(workspace__in=user_new_workspaces).delete()

        user = super(self.__class__, self).update(instance, validated_data)
        if 'password' in validated_data:
            user.set_password(validated_data['password'])
            user.save()
        return user

    def create(self, validated_data):
        user_role_workspaces_data = validated_data.pop('user_role_workspaces')
        user = User.objects.create(**validated_data)
        for user_role_workspace_data in user_role_workspaces_data:
            UserRoleWorkspace.objects.create(
                user=user, **user_role_workspace_data
            )
        if 'password' in validated_data:
            user.set_password(validated_data['password'])
            user.save()
        return user


class UsersFilter(filters.FilterSet):
    class Meta:
        fields = {
            "id": ["exact", "in"],
            "first_name": ["exact", "in", "contains"],
            "generic_groups": ["in"],
            "last_name": ["exact", "in", "contains"],
            "email": ["exact", "in", "contains"],
            "username": ["exact", "in", "contains"],
            "created": ["gt", "gte", "lt", "lte"],
            "modified": ["gt", "gte", "lt", "lte"],
        }
        model = User

class UsersViewSet(MunityViewSet):
    serializer_class = UserSerializer
    filterset_class = UsersFilter

    def destroy(self, request, pk=None, workspace_pk=None):
        # we suppose that user as permission USER DELETE for current workspace
        # but we need to perform some sanity check
        if not request.user.is_superuser:
            accessible_workspaces = UserRoleWorkspace.objects.filter(user=self.request.user)
            workspace_slugs = []
            for accessible_workspace in accessible_workspaces:
                workspace_slugs.append(accessible_workspace.workspace)
            user_to_delete = self.serializer_class.Meta.model.objects.filter(
                id=pk,
                user_role_workspaces__in=UserRoleWorkspace.objects.filter(workspace__in=workspace_slugs)
            ).first()
            if not user_to_delete:
                raise PermissionDenied({"message":"You cannot remove user, you don't have any workspace in common"})
        response = super().destroy(request, pk=pk , workspace_pk=workspace_pk)
        return response

    def create(self, request, workspace_pk=None):
        # we suppose that user as permission USER CREATE for current workspace
        # but we need to perform some sanity check
        if not request.user.is_superuser:
            if request.data.get('is_superuser'):
                raise PermissionDenied({"message":"You cannot create superuser"})

            accessible_workspaces = UserRoleWorkspace.objects.filter(user=self.request.user)
            workspace_slugs = []
            for accessible_workspace in accessible_workspaces:
                workspace_slugs.append(accessible_workspace.workspace.slug)
            user_role_workspaces = request.data.get('user_role_workspaces')
            if not user_role_workspaces:
                raise PermissionDenied({"message":"Cannot create user without roles"})
            for user_role_workspace in user_role_workspaces:
                if user_role_workspace.get('workspace') not in workspace_slugs:
                    raise PermissionDenied({"message":"You don't have permission to add user on this workspace"})


        if 'generic_groups' in request.data:
            generic_groups = request.data.pop('generic_groups')
            response = super().create(request, workspace_pk)
            new_user = User.objects.filter(id=response.data.get("id")).first()
            new_user.generic_groups.set(generic_groups)
            new_user.save()
            return Response(UserSerializer(new_user).data, status=status.HTTP_201_CREATED)

        return super().create(request, workspace_pk)

    def update(self, request, workspace_pk=None, pk=None, partial=False):
        # we suppose that user as permission USER UPDATE for current workspace
        # but we need to perform some sanity check
        if not request.user.is_superuser:
            if request.data.get('is_superuser'):
                raise PermissionDenied({"message":"You cannot grant superuser privilege"})

            edited_user = self.get_queryset().filter(pk=pk).first()
            if edited_user is None:
                raise PermissionDenied({"message":"You cannot see this user"})

            if edited_user.is_superuser:
                raise PermissionDenied({"message":"You cannot edit superuser"})

            # update role if needed
            if 'user_role_workspaces' in request.data:
                accessible_workspaces = UserRoleWorkspace.objects.filter(user=self.request.user)
                workspace_slugs = []
                for accessible_workspace in accessible_workspaces:
                    workspace_slugs.append(accessible_workspace.workspace.slug)

                user_role_workspaces = request.data.get('user_role_workspaces')
                for user_role_workspace in user_role_workspaces:
                    if user_role_workspace.get('workspace') not in workspace_slugs:
                        raise PermissionDenied({"message":"You don't have permission to change this access"})

        response = super().update(request, pk=pk, workspace_pk=workspace_pk, partial=partial)
        return response

    # since user access workspaces by role, we have to adapt access permission through role and not workpace FK
    def get_queryset(self):
        model = self.serializer_class.Meta.model
        queryset = model.objects.filter(disabled=False)
        if "workspace_pk" in self.kwargs:
            # super user see all workspace users
            if self.request.user.is_superuser:
                return queryset.filter(Q(id=self.request.user.id) | Q(user_role_workspaces__workspace=self.kwargs["workspace_pk"])).distinct()
            else:
                # getting accessible workspaces
                accessible_workspaces = UserRoleWorkspace.objects.filter(user=self.request.user)
                workspace_slugs = []
                for accessible_workspace in accessible_workspaces:
                    workspace_slugs.append(accessible_workspace.workspace)
                # get all related roles
                return queryset.filter(
                    user_role_workspaces__in=UserRoleWorkspace.objects.filter(workspace__in=workspace_slugs).filter(workspace__in=[self.kwargs["workspace_pk"]])
                )
        # WE ARE ON OVERMIND!
        else:
            # super user see all overmind users
            if self.request.user.is_superuser:
                # return model.objects.filter(Q(is_superuser=True) | Q(has_overmind_access=True))
                return queryset.all()
            # staff see user from his workspaces
            if self.request.user.has_overmind_access:
                # getting accessible workspaces
                accessible_workspaces = UserRoleWorkspace.objects.filter(user=self.request.user)
                workspace_slugs = []
                for accessible_workspace in accessible_workspaces:
                    workspace_slugs.append(accessible_workspace.workspace)
                # get all related roles
                return queryset.filter(
                    user_role_workspaces__in=UserRoleWorkspace.objects.filter(workspace__in=workspace_slugs)
                )
            # users see nothing
            return None



# Generated by Django 3.2.7 on 2021-12-06 13:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0020_delete_usersviewset'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='phone_number',
            field=models.CharField(blank=True, default=None, max_length=20, null=True),
        ),
    ]



from django_filters import rest_framework as filters
from munity.views import MunityViewSet
from rest_framework import serializers

from .models import GenericGroup

class GenericGroupSerializer(serializers.ModelSerializer):
    class Meta:
        fields = [
            "id",
            "label",
            "generic_groups",
            "workspace",
            "created",
            "modified",
        ]
        model = GenericGroup


class GenericGroupsFilter(filters.FilterSet):
    class Meta:
        fields = {
            "id": ["exact", "in"],
            "label": ["exact", "in", "contains"],
            "generic_groups": ["in"],
            "created": ["gt", "gte", "lt", "lte"],
            "modified": ["gt", "gte", "lt", "lte"],
        }
        model = GenericGroup


class GenericGroupsViewSet(MunityViewSet):
    serializer_class = GenericGroupSerializer
    filterset_class = GenericGroupsFilter



import uuid

# from .groups.models import Group

from django.db import models
from django_extensions.db.models import TimeStampedModel
from django.db.models.signals import post_save


class MunityModel(TimeStampedModel):
    """
    Default base class for munity models
    use uuid for ids
    add creation and update date
    """

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    generic_groups = models.ManyToManyField('generic_groups.GenericGroup', blank=True)
    disabled = models.BooleanField(default=False)
    workspace = models.ForeignKey(
        "workspaces.Workspace", to_field="slug", on_delete=models.CASCADE, null=True, blank=True
    )

    class Meta:
        abstract = True

    def __str__(self):
        return f"{self.id}@{self.workspace or 'overmind'}"



# Generated by Django 3.2.7 on 2021-11-18 22:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0014_auto_20211118_2146'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='user',
            name='roles',
        ),
        migrations.AddField(
            model_name='user',
            name='workspace_roles',
            field=models.ManyToManyField(related_name='workspace_roles', to='users.UserRoleWorkspace'),
        ),
    ]



# Generated by Django 3.2.7 on 2021-12-15 16:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('generic_groups', '0006_alter_genericgroup_workspace'),
    ]

    operations = [
        migrations.AddField(
            model_name='genericgroup',
            name='disabled',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.2.7 on 2021-11-21 19:07

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('generic_groups', '0006_alter_genericgroup_workspace'),
        ('workspaces', '0009_alter_workspace_slug'),
        ('users', '0015_auto_20211118_2214'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='user',
            name='workspace',
        ),
        migrations.RemoveField(
            model_name='user',
            name='workspace_roles',
        ),
        migrations.AlterField(
            model_name='userroleworkspace',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user_role_workspaces', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterUniqueTogether(
            name='userrolegenericgroup',
            unique_together={('user', 'group')},
        ),
        migrations.AlterUniqueTogether(
            name='userroleworkspace',
            unique_together={('user', 'workspace')},
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-17 22:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('workspaces', '0007_delete_environmentconfig'),
    ]

    operations = [
        migrations.AlterField(
            model_name='workspace',
            name='db_connection',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]



from django_filters import rest_framework as filters
from rest_framework import serializers, viewsets
from ..users.models import UserRoleWorkspace
from rest_framework.exceptions import PermissionDenied


from .models import Workspace


class WorkspaceSerializer(serializers.ModelSerializer):
    class Meta:
        fields = [
            "slug",
            "name",
            "db_connection",
            "created",
            "modified",
        ]
        model = Workspace


class WorkspacesFilter(filters.FilterSet):
    class Meta:
        fields = {
            "slug": ["exact", "in"],
            "name": ["exact", "in"],
            "created": ["gt", "gte", "lt", "lte"],
            "modified": ["gt", "gte", "lt", "lte"],
        }
        model = Workspace

class WorkspacesViewSet(viewsets.ModelViewSet):
    serializer_class = WorkspaceSerializer
    filterset_class = WorkspacesFilter

    # since user access workspaces by role, we have to adapt access permission through role and not workpace FK
    def get_queryset(self):
        model = self.serializer_class.Meta.model
        queryset = model.objects.filter(disabled=False)
        # super user see all
        if self.request.user.is_superuser:
            return queryset.all()
        # users see only there workspaces
        accessible_workspaces = UserRoleWorkspace.objects.filter(user=self.request.user)
        workspace_slugs = []
        for accessible_workspace in accessible_workspaces:
            workspace_slugs.append(accessible_workspace.workspace)
        return queryset.filter(slug__in=workspace_slugs)




from django_extensions.db.models import TimeStampedModel
from django.db import models


class Workspace(TimeStampedModel):
    class Meta:
        ordering = ("slug",)

    disabled = models.BooleanField(default=False)
    slug = models.SlugField(primary_key=True, max_length=50, unique=True)
    name = models.CharField(max_length=258, blank=True, null=False, default="")
    db_connection = models.CharField(max_length=100, blank=True, null=True, default=True)

    def get_absolute_url(self):
        return f"/workspaces/{self.slug}"

    def __str__(self):
        return self.slug



# Generated by Django 3.2.7 on 2021-10-17 21:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('workspaces', '0006_auto_20211017_2105'),
    ]

    operations = [
        migrations.DeleteModel(
            name='EnvironmentConfig',
        ),
    ]



from rest_framework import serializers, viewsets
from django_filters import rest_framework as filters
from django.db.models import Q

from ..views import MunityViewSet

from .models import Settings

class SettingsSerializer(serializers.ModelSerializer):
    class Meta:
        fields = [
            "key",
            "value",
            "workspace",
            "created",
            "modified",
        ]
        model = Settings


class SettingsFilter(filters.FilterSet):
    class Meta:
        fields = {
            "workspace": ["exact", "in"],
            "key": ["exact", "in"],
            "created": ["gt", "gte", "lt", "lte"],
            "modified": ["gt", "gte", "lt", "lte"],
        }
        model = Settings


class SettingsViewSet(MunityViewSet):
    serializer_class = SettingsSerializer
    filterset_class = SettingsFilter


from django.contrib import admin
from .models import Settings

# @admin.register(Settings)
# class SettingsAdmin(admin.ModelAdmin):
#     list_display = ("key", "value", "workspace")
#     list_editable = ("value",)
#     list_filter = ("key", "workspace")


from django.core.management.base import BaseCommand, CommandError
from django.db.models.query_utils import Q
from munity.authorization.models import Role, Permission

class Command(BaseCommand):
    help = 'Initiate Munity with basic authorization'

    def handle(self, *args, **options):
        self.stdout.write("Adding Owner, Admin and User role")
        # (owner_role, _) = Role.objects.update_or_create(
        #     name="Owner"
        # )

        (admin_role, _) = Role.objects.update_or_create(
            name="Admin"
        )

        (user_role, _) = Role.objects.update_or_create(
            name="User"
        )

        # godmode=1
        # owner_role.permissions.set(Permission.objects.all())
        # can do everything but edit/create workspace
        admin_role.permissions.set(Permission.objects.exclude(
            Q(action=["delete", "update", "create"]) &
            Q(ressource__model__in=["workspace"])
        ))
        # readonly
        user_role.permissions.set(Permission.objects.filter(action__in=["list", "retrieve"]))



import uuid
from django_extensions.db.models import TimeStampedModel
from django.db import models

from ..models import MunityModel

def generate_filename(self, filename):
    url = "%s/%s" % (self.workspace if self.workspace else 'overmind', f"{uuid.uuid4()}.{filename.split('.')[-1]}")
    return url

class File(MunityModel):
    name = models.CharField(max_length=255, blank=True)
    type = models.CharField(max_length=32, blank=True)
    size = models.BigIntegerField(blank=True)
    file = models.FileField(upload_to=generate_filename)

    class Meta:
        ordering = ("-created",)

    def __str__(self):
        return f"{self.name}"



# Generated by Django 3.2.7 on 2021-10-17 21:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0003_rename_related_workspace_user_workspace'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='user',
            name='workspace',
        ),
    ]



from rest_framework.response import Response
from rest_framework_simplejwt.serializers import TokenObtainPairSerializer
from rest_framework_simplejwt.views import TokenObtainPairView

from django.contrib.contenttypes.models import ContentType

from ..records.models import Record
from ..users.models import User
from ..users.views import UserSerializer

import jwt


class MyTokenObtainPairSerializer(TokenObtainPairSerializer):
    @classmethod
    def get_token(cls, user):
        token = super().get_token(user)
        user.password = "********"
        token["user_at_first_login"] = UserSerializer(user).data

        return token

class MyTokenObtainPairView(TokenObtainPairView):
    serializer_class = MyTokenObtainPairSerializer

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)

        response = super().post(request, args, kwargs)
        # we assume that credentials are correct if parent class do not trigg errors

        ctype = ContentType.objects.get_for_model(User)
        accesst_token = response.data.get("access")
        data = jwt.decode(accesst_token, options={"verify_signature": False})

        Record.objects.create(
            action = "login",
            product_object_id = data["user_id"],
            product_content_type = ctype,
            user_id = data["user_id"]
        )

        return response


# Generated by Django 3.2.7 on 2021-10-17 20:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('workspaces', '0003_auto_20211017_2031'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='workspace',
            name='id',
        ),
        migrations.AlterField(
            model_name='workspace',
            name='slug',
            field=models.CharField(max_length=50, primary_key=True, serialize=False, unique=True),
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-17 17:13

from django.db import migrations, models
import django_extensions.db.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='GenericGroup',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('label', models.CharField(max_length=255)),
                ('generic_groups', models.ManyToManyField(blank=True, to='generic_groups.GenericGroup')),
            ],
            options={
                'verbose_name': 'Generic group',
                'verbose_name_plural': 'Generic groups',
            },
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-17 20:17

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('workspaces', '0002_auto_20211017_2017'),
        ('records', '0002_record_user'),
    ]

    operations = [
        migrations.AddField(
            model_name='record',
            name='related_workspace',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='workspaces.workspace'),
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-17 17:13

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('workspaces', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Permission',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('action', models.CharField(max_length=100)),
                ('ressource', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='munity_permissions', to='contenttypes.contenttype')),
            ],
            options={
                'unique_together': {('ressource', 'action')},
            },
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('permissions', models.ManyToManyField(blank=True, to='authorization.Permission')),
                ('workspace', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='workspaces.workspace')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.2.7 on 2021-12-02 12:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('workspaces', '0009_alter_workspace_slug'),
        ('records', '0008_remove_record_model_name'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='record',
            options={'get_latest_by': 'modified'},
        ),
        migrations.RenameField(
            model_name='record',
            old_name='next_value',
            new_name='diff_value',
        ),
        migrations.AddField(
            model_name='record',
            name='action',
            field=models.CharField(default='', max_length=64),
        ),
        migrations.AlterField(
            model_name='record',
            name='workspace',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='workspaces.workspace'),
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-17 21:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('workspaces', '0004_auto_20211017_2049'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='environmentconfig',
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name='environmentconfig',
            name='workspace',
        ),
    ]



# Generated by Django 3.2.7 on 2021-12-02 21:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0019_user_has_overmind_access'),
    ]

    operations = [
        migrations.DeleteModel(
            name='UsersViewSet',
        ),
    ]



# Generated by Django 3.2.7 on 2021-11-24 16:08

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields
import munity.files.models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('workspaces', '0009_alter_workspace_slug'),
    ]

    operations = [
        migrations.CreateModel(
            name='File',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('name', models.CharField(blank=True, max_length=255)),
                ('type', models.CharField(blank=True, max_length=32)),
                ('size', models.BigIntegerField(blank=True)),
                ('file', models.FileField(upload_to=munity.files.models.generate_filename)),
                ('workspace', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='workspaces.workspace')),
            ],
            options={
                'ordering': ('-created',),
            },
        ),
    ]



# Generated by Django 3.2.7 on 2021-12-02 12:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('records', '0009_auto_20211202_1229'),
    ]

    operations = [
        migrations.AlterField(
            model_name='record',
            name='diff_value',
            field=models.JSONField(null=True),
        ),
        migrations.AlterField(
            model_name='record',
            name='previous_value',
            field=models.JSONField(null=True),
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-17 20:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('workspaces', '0002_auto_20211017_2017'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='workspace',
            options={'get_latest_by': 'modified'},
        ),
        migrations.RenameField(
            model_name='environmentconfig',
            old_name='related_workspace',
            new_name='workspace',
        ),
        migrations.RemoveField(
            model_name='workspace',
            name='related_workspace',
        ),
        migrations.AlterField(
            model_name='workspace',
            name='slug',
            field=models.CharField(max_length=50, unique=True),
        ),
        migrations.AlterUniqueTogether(
            name='environmentconfig',
            unique_together={('key', 'workspace')},
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-19 09:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('workspaces', '0008_alter_workspace_db_connection'),
        ('authorization', '0005_auto_20211017_2105'),
    ]

    operations = [
        migrations.AlterField(
            model_name='permission',
            name='workspace',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='workspaces.workspace'),
        ),
        migrations.AlterField(
            model_name='role',
            name='workspace',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='workspaces.workspace'),
        ),
    ]



from django.contrib import admin

from .models import Permission, Role


@admin.register(Permission)
class PermissionAdmin(admin.ModelAdmin):
    list_display = ("ressource_name", "action")

    @admin.display()
    def ressource_name(self, obj):
        return obj.ressource.model

    list_filter = ("action", "ressource")


@admin.register(Role)
class RoleAdmin(admin.ModelAdmin):
    list_display = ("name",)



# Generated by Django 3.2.7 on 2021-10-19 17:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0011_user_testt'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='user',
            name='testt',
        ),
    ]



from django_filters import rest_framework as filters
from rest_framework import serializers, viewsets
from django.contrib.contenttypes.models import ContentType

from .models import Permission, Role

#################################################
# Permission
#################################################


class RessourceSerializer(serializers.ModelSerializer):
    class Meta:
        fields = "__all__"
        model = ContentType

class PermissionSerializer(serializers.ModelSerializer):
    class Meta:
        fields = [
            "id",
            "ressource",
            "action",
        ]
        model = Permission
    ressource = RessourceSerializer()

class PermissionsFilter(filters.FilterSet):
    class Meta:
        fields = {
            "id": ["exact", "in"],
            "ressource": [
                "exact",
                "in",
            ],
            "action": [
                "exact",
                "in",
            ],
            "created": ["gt", "gte", "lt", "lte"],
            "modified": ["gt", "gte", "lt", "lte"],
        }
        model = Permission


class PermissionsViewSet(viewsets.ModelViewSet):
    queryset = Permission.objects.all()
    serializer_class = PermissionSerializer
    filterset_class = PermissionsFilter


#################################################
# Role
#################################################


class RoleSerializer(serializers.ModelSerializer):
    class Meta:
        fields = ["id", "name", "permissions"]
        model = Role
    permissions = PermissionSerializer(many=True)


class RolesFilter(filters.FilterSet):
    class Meta:
        fields = {
            "id": ["exact", "in"],
            "name": [
                "exact",
                "in",
            ],
            "permissions": ["exact", "in"],
            "created": ["gt", "gte", "lt", "lte"],
            "modified": ["gt", "gte", "lt", "lte"],
        }
        model = Role


class RolesViewSet(viewsets.ModelViewSet):
    queryset = Role.objects.all()
    serializer_class = RoleSerializer
    filterset_class = RolesFilter



from django.contrib.contenttypes.models import ContentType
from rest_framework import viewsets
from rest_framework.response import Response
from rest_framework import status
from deepdiff import DeepDiff

from .workspaces.models import Workspace
from .records.models import Record

from .utils import UUIDEncoder

import json

# Create your views here.
class MunityViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        model = self.serializer_class.Meta.model

        # filter request to current model workspace
        queryset = model.objects.filter(disabled=False)
        if "workspace_pk" in self.kwargs:
            return queryset.filter(workspace=self.kwargs["workspace_pk"])
        return queryset.all()

    def destroy(self, request, pk=None, workspace_pk=None):
        # store model that will be deleted
        deleted_model = self.serializer_class.Meta.model.objects.get(pk=pk)

        # get its content type
        ctype = ContentType.objects.get_for_model(deleted_model)

        # destroy item
        response = super().destroy(request=request, workspace_pk=workspace_pk, pk=pk)

        # record the vicious action!
        record = Record.objects.create(
            previous_value = None,
            diff_value = None,
            action = 'delete',
            workspace = Workspace.objects.filter(slug=workspace_pk).first(),
            user = request.user,
            product_object_id = pk,
            product_content_type = ctype
        )
        record.save()
        return response

    def create(self, request, workspace_pk=None):
        # getting workspace
        current_workspace = Workspace.objects.filter(slug=workspace_pk).first()

        # add new workspace params
        request.data['workspace'] = current_workspace

        # create instance, link groups, get response
        if 'generic_groups' in request.data.keys():
            generic_groups = request.data.pop('generic_groups')
            _response = super().create(request, workspace_pk)
            instance = self.serializer_class.Meta.model.objects.filter(id=_response.data.get("id")).first()
            instance.generic_groups.set(generic_groups)
            instance.save()
            response = Response(self.serializer_class(instance).data, status=status.HTTP_201_CREATED)
        else:
            response = super().create(request, workspace_pk)

        # storing record
        model_id = dict(response.data).get('id')
        new_model = self.serializer_class.Meta.model.objects.get(pk=model_id)
        ctype = ContentType.objects.get_for_model(new_model)
        record = Record.objects.create(
            previous_value = None,
            diff_value = None,
            action = 'create',
            workspace = current_workspace,
            user = request.user,
            product_object_id = model_id,
            product_content_type = ctype
        )
        record.save()
        return response

    def update(self, request, pk=None, workspace_pk=None, partial=False):
        # for now we API cannot move item across workspace, we dont add workspace to serializer

        # keep a record for old data
        old_model = self.serializer_class.Meta.model.objects.get(pk=pk)

        # update model
        response = super().update(request, pk=pk, workspace_pk=workspace_pk, partial=partial)

        # get new data
        new_model = self.serializer_class.Meta.model.objects.get(pk=pk)

        # calculate diff
        diff = DeepDiff(
            old_model,
            new_model,
            exclude_paths=["root.modified"]
        )
        ctype = ContentType.objects.get_for_model(old_model)

        # store record if change was made
        if diff:
            record = Record.objects.create(
                previous_value = json.dumps(self.serializer_class(old_model).data, cls=UUIDEncoder),
                diff_value = diff.get("values_changed"),
                action = 'update',
                workspace = Workspace.objects.filter(slug=workspace_pk).first(),
                user = request.user,
                product_object_id = pk,
                product_content_type = ctype
            )
            record.save()
        return response




# Generated by Django 3.2.7 on 2021-12-02 17:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('workspaces', '0009_alter_workspace_slug'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='workspace',
            options={'ordering': ('slug',)},
        ),
    ]



# Generated by Django 3.2.7 on 2021-11-22 16:22

from django.db import migrations
import django.db.models.expressions


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0016_auto_20211121_1907'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='user',
            options={'ordering': (django.db.models.expressions.Func(django.db.models.expressions.F('username'), function='Lower'),)},
        ),
    ]



# Generated by Django 3.2.7 on 2021-12-02 17:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0018_user_avatar'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='has_overmind_access',
            field=models.BooleanField(blank=True, default=False),
        ),
    ]



# from rest_framework_extensions.routers import ExtendedDefaultRouter as DefaultRouter
from rest_framework_nested import routers as nested_routers

from .authorization.views import PermissionsViewSet, RolesViewSet
from .users.views import UsersViewSet
from .generic_groups.views import GenericGroupsViewSet
from .workspaces.views import WorkspacesViewSet
from .settings.views import SettingsViewSet
from .files.views import FileViewSet

overmind_router = nested_routers.SimpleRouter()
overmind_router.register(r"users", UsersViewSet, basename="user")
overmind_router.register(r"workspaces", WorkspacesViewSet, basename="workspace")
overmind_router.register(r"roles", RolesViewSet, basename="role")
overmind_router.register(r"generic_groups", GenericGroupsViewSet, basename="group")
overmind_router.register(r"permissions", PermissionsViewSet, basename="permission")
overmind_router.register(r"settings", SettingsViewSet, basename="settings")
overmind_router.register(r"files", FileViewSet, basename="files")

workspace_router = nested_routers.NestedSimpleRouter(overmind_router, r'workspaces', lookup='workspace')
workspace_router.register(r"users", UsersViewSet, basename="workspace-user")
workspace_router.register(r"roles", RolesViewSet, basename="workspace-user")
workspace_router.register(r"generic_groups", GenericGroupsViewSet, basename="workspace-user")
workspace_router.register(r"permissions", PermissionsViewSet, basename="workspace-user")
workspace_router.register(r"settings", SettingsViewSet, basename="setting")
workspace_router.register(r"files", FileViewSet, basename="files")
